local ADDON_NAME = ...
local _G = getfenv(0)
local xo, yo = 8, 3 -- X-offset, Y-offset
local equiped = {} -- Table to store equiped items
local setting -- Setting on how much we show
local socketsTable = {
	-- /dump string.split(":", GetInventoryItemLink("player", i))
	-- WoD Sockets
	[523] = true, -- Dungeon
	[563] = true, -- Normal Raid
	[564] = true, -- Heroic Raid
	[565] = true, -- Mythic Raid
	-- Prismatic Sockets in 7.0, how many are there?
	[1808] = true, -- From Heroic Dungeons and T19 Normal Raids
	[3386] = true, -- From Vendor
	[3458] = true, -- Legendary item with socket?
	--[3459] = true, -- Legendary stuff, but no idea what this does
	-- Are these sockets?
	--[[[3] = true,
	[497] = true,
	[572] = true,]]
}
local f = CreateFrame("Frame", nil, _G.PaperDollFrame) -- iLvel number frame
local g -- iLvel number for Inspect frame
f:RegisterEvent("ADDON_LOADED")
f:RegisterEvent("PLAYER_LOGIN")

-- Tooltip and scanning by Phanx @ http://www.wowinterface.com/forums/showthread.php?p=271406
local S_ITEM_LEVEL = "^" .. gsub(ITEM_LEVEL, "%%d", "(%%d+)")
local S_UPGRADE_LEVEL = "^" .. gsub(ITEM_UPGRADE_TOOLTIP_FORMAT, "%%d", "(%%d+)")
local S_HEIRLOOM_LEVEL = "^" .. gsub(HEIRLOOM_UPGRADE_TOOLTIP_FORMAT, "%%d", "(%%d+)")

local scantip = CreateFrame("GameTooltip", "iLvlScanningTooltip", nil, "GameTooltipTemplate")
scantip:SetOwner(UIParent, "ANCHOR_NONE")

local function _getRealItemLevel(slotId, unit)
	local realItemLevel, currentUpgradeLevel, maxUpgradeLevel
	local hasItem = scantip:SetInventoryItem(unit, slotId)
	if not hasItem then return nil end -- With this we don't get ilvl for offhand if we equip 2h weapon

	for i = 2, scantip:NumLines() do -- Line 1 is always the name so you can skip it.
		local text = _G["iLvlScanningTooltipTextLeft"..i]:GetText()
		if text and text ~= "" then
			realItemLevel = realItemLevel or strmatch(text, S_ITEM_LEVEL)
			if not (currentUpgradeLevel or maxUpgradeLevel) then
				currentUpgradeLevel, maxUpgradeLevel = strmatch(text, S_UPGRADE_LEVEL)
				if not (currentUpgradeLevel or maxUpgradeLevel) then
					currentUpgradeLevel, maxUpgradeLevel = strmatch(text, S_HEIRLOOM_LEVEL)
				end
			end

			if realItemLevel and currentUpgradeLevel and maxUpgradeLevel then
				return realItemLevel, tonumber(currentUpgradeLevel), tonumber(maxUpgradeLevel)
			end
		end
	end

	return realItemLevel
end

local function _updateItems(unit, frame)
	for i = 1, 17 do -- Only check changed player items or items without ilvl text, skip the shirt (4) and always update Inspects
		local itemLink = GetInventoryItemLink(unit, i)
		if i ~= 4 and ((frame == f and (equiped[i] ~= itemLink or frame[i]:GetText() == nil)) or frame == g) then
			if frame == f then
				equiped[i] = itemLink
			end

			local realItemLevel, currentUpgradeLevel, maxUpgradeLevel = _getRealItemLevel(i, unit)
			local upgradeString, enchantString, gemString = "", "", ""
			local _, enchantID, upgradeTypeID, numBonuses, affixes

			if setting >= 1 then
				if currentUpgradeLevel and maxUpgradeLevel and currentUpgradeLevel < maxUpgradeLevel then
					upgradeString = "|TInterface\\PetBattles\\BattleBar-AbilityBadge-Strong:0:0:0:0:32:32:2:30:2:30|t"
				end

				if setting == 2 then
					if itemLink then
						_, _, enchantID, _, _, _, _, _, _, _, _, upgradeTypeID, _, numBonuses, affixes = strsplit(":", itemLink, 15)
					end
					enchantID = tonumber(enchantID); numBonuses = tonumber(numBonuses); upgradeTypeID = tonumber(upgradeTypeID); realItemLevel = realItemLevel or ""

					if i == 2 or i == 3 or i == 11 or i == 12 or i == 15 then
						-- Neck, Shoulders, Finger0, Finger1, Chest
						if enchantID and enchantID > 0 then
							enchantString = "|T136244:0:0:0:0:32:32:2:30:2:30|t"
						elseif itemLink then
							enchantString = "|T136244:0:0:0:0:32:32:2:30:2:30:221:0:0|t"
						end
					end

					if i == 16 and upgradeTypeID == 256 then -- Main Hand, Artifact Weapon
						for b = 1, 3 do
							local _, gemLink = GetItemGem(itemLink, b)
							if gemLink and gemLink ~= "" then
								local _, _, _, _, _, _, _, _, _, t = GetItemInfo(gemLink)
								if t and t > 0 then
									gemString = gemString.."|T"..t..":0:0:0:0:32:32:2:30:2:30|t"
								end
							else
								gemString = gemString.."|TInterface\\ItemSocketingFrame\\UI-EmptySocket-Red:0:0:0:0:32:32:2:30:2:30|t"
							end
						end							
					elseif numBonuses and numBonuses > 0 then
						for b = 1, numBonuses do
							local bonusID = select(b, strsplit(":", affixes))
							if socketsTable[tonumber(bonusID)] then
								local _, gemLink = GetItemGem(itemLink, 1)
								if gemLink and gemLink ~= "" then
									local _, _, _, _, _, _, _, _, _, t = GetItemInfo(gemLink)
									if t and t > 0 then
										gemString = gemString.."|T"..t..":0:0:0:0:32:32:2:30:2:30|t"
									end
								else
									gemString = gemString.."|TInterface\\ItemSocketingFrame\\UI-EmptySocket-Red:0:0:0:0:32:32:2:30:2:30|t"
								end
							end
						end
					end
				end
			end

			realItemLevel = realItemLevel or ""

			if setting == 2 then
				if i <= 5 or i == 15 or i == 9 then -- Left side
					frame[i]:SetFormattedText("%s%s%s%s", realItemLevel, enchantString, gemString, upgradeString)
				elseif i == 16 then -- MainHand
					if gemString ~= "" then
						frame[i]:SetFormattedText("%s%s\n%s", upgradeString, realItemLevel, gemString)
						if frame == f then
							frame[i]:SetWidth(_G.CharacterMainHandSlot:GetWidth() + 2) -- Fix for 3 gems in weapon
						else
							frame[i]:SetWidth(_G.InspectMainHandSlot:GetWidth() + 2) -- Fix for 3 gems in weapon
						end
					else
						frame[i]:SetFormattedText("%s%s", upgradeString, realItemLevel)
						if frame == f then
							frame[i]:SetWidth(_G.CharacterMainHandSlot:GetWidth())
						else
							frame[i]:SetWidth(_G.InspectMainHandSlot:GetWidth())
						end
					end
				elseif i == 17 then -- OffHand
					if gemString ~= "" then
						frame[i]:SetFormattedText("%s%s\n%s", realItemLevel, upgradeString, gemString)
					else
						frame[i]:SetFormattedText("%s%s", realItemLevel, upgradeString)
					end
				else -- Right side
					frame[i]:SetFormattedText("%s%s%s%s", upgradeString, gemString, enchantString, realItemLevel)
				end
			elseif setting == 1 then
				if i <= 5 or i == 15 or i == 9 or i == 17 then -- Left side
					frame[i]:SetText(realItemLevel..upgradeString)
				else
					frame[i]:SetText(upgradeString..realItemLevel)
				end
			else
				frame[i]:SetText(realItemLevel)
			end
		end
	end
end

local function _createStrings()
	local function _stringFactory(parent, myPoint, parentPoint, x, y)
		local s = f:CreateFontString(nil, "OVERLAY", "GameFontNormalOutline")
		s:SetPoint(myPoint, parent, parentPoint, x or 0, y or 0)

		return s
	end

	f:SetFrameLevel(_G.CharacterHeadSlot:GetFrameLevel())

	f[1] = _stringFactory(_G.CharacterHeadSlot, "LEFT", "RIGHT", xo)
	f[2] = _stringFactory(_G.CharacterNeckSlot, "LEFT", "RIGHT", xo)
	f[3] = _stringFactory(_G.CharacterShoulderSlot, "LEFT", "RIGHT", xo)
	f[15] = _stringFactory(_G.CharacterBackSlot, "LEFT", "RIGHT", xo)
	f[5] = _stringFactory(_G.CharacterChestSlot, "LEFT", "RIGHT", xo)
	f[9] = _stringFactory(_G.CharacterWristSlot, "LEFT", "RIGHT", xo)

	f[10] = _stringFactory(_G.CharacterHandsSlot, "RIGHT", "LEFT", -xo)
	f[6] = _stringFactory(_G.CharacterWaistSlot, "RIGHT", "LEFT", -xo)
	f[7] = _stringFactory(_G.CharacterLegsSlot, "RIGHT", "LEFT", -xo)
	f[8] = _stringFactory(_G.CharacterFeetSlot, "RIGHT", "LEFT", -xo)
	f[11] = _stringFactory(_G.CharacterFinger0Slot, "RIGHT", "LEFT", -xo)
	f[12] = _stringFactory(_G.CharacterFinger1Slot, "RIGHT", "LEFT", -xo)
	f[13] = _stringFactory(_G.CharacterTrinket0Slot, "RIGHT", "LEFT", -xo)
	f[14] = _stringFactory(_G.CharacterTrinket1Slot, "RIGHT", "LEFT", -xo)

	f[16] = _stringFactory(_G.CharacterMainHandSlot, "BOTTOM", "TOP", 2, yo)
	f[17] = _stringFactory(_G.CharacterSecondaryHandSlot, "BOTTOM", "TOP", 2, yo)

	f:Hide()
end

local function _createGStrings()
	local function _stringFactory(parent, myPoint, parentPoint, x, y)
		local s = g:CreateFontString(nil, "OVERLAY", "GameFontNormalOutline")
		s:SetPoint(myPoint, parent, parentPoint, x or 0, y or 0)

		return s
	end

	g:SetFrameLevel(_G.InspectHeadSlot:GetFrameLevel())

	g[1] = _stringFactory(_G.InspectHeadSlot, "LEFT", "RIGHT", xo)
	g[2] = _stringFactory(_G.InspectNeckSlot, "LEFT", "RIGHT", xo)
	g[3] = _stringFactory(_G.InspectShoulderSlot, "LEFT", "RIGHT", xo)
	g[15] = _stringFactory(_G.InspectBackSlot, "LEFT", "RIGHT", xo)
	g[5] = _stringFactory(_G.InspectChestSlot, "LEFT", "RIGHT", xo)
	g[9] = _stringFactory(_G.InspectWristSlot, "LEFT", "RIGHT", xo)

	g[10] = _stringFactory(_G.InspectHandsSlot, "RIGHT", "LEFT", -xo)
	g[6] = _stringFactory(_G.InspectWaistSlot, "RIGHT", "LEFT", -xo)
	g[7] = _stringFactory(_G.InspectLegsSlot, "RIGHT", "LEFT", -xo)
	g[8] = _stringFactory(_G.InspectFeetSlot, "RIGHT", "LEFT", -xo)
	g[11] = _stringFactory(_G.InspectFinger0Slot, "RIGHT", "LEFT", -xo)
	g[12] = _stringFactory(_G.InspectFinger1Slot, "RIGHT", "LEFT", -xo)
	g[13] = _stringFactory(_G.InspectTrinket0Slot, "RIGHT", "LEFT", -xo)
	g[14] = _stringFactory(_G.InspectTrinket1Slot, "RIGHT", "LEFT", -xo)

	g[16] = _stringFactory(_G.InspectMainHandSlot, "BOTTOM", "TOP", 2, yo)
	g[17] = _stringFactory(_G.InspectSecondaryHandSlot, "BOTTOM", "TOP", 2, yo)

	g:Hide()
end

local function OnEvent(self, event, ...) -- Event handler
	if event == "ADDON_LOADED" and (...) == "Blizzard_InspectUI" then
		self:UnregisterEvent(event)

		g = CreateFrame("Frame", nil, _G.InspectPaperDollFrame) -- iLevel number frame for Inspect
		_createGStrings()
		_createGStrings = nil

		_G.InspectPaperDollFrame:HookScript("OnShow", function(self)
			g:SetFrameLevel(_G.InspectHeadSlot:GetFrameLevel())
			f:RegisterEvent("INSPECT_READY")
			_updateItems("target", g)
			g:Show()
		end)

		_G.InspectPaperDollFrame:HookScript("OnHide", function(self)
			f:UnregisterEvent("INSPECT_READY")
			g:Hide()
		end)
	elseif event == "PLAYER_LOGIN" then
		self:UnregisterEvent(event)

		if type(iLevelSetting) ~= "number" then
			iLevelSetting = 2
		end
		setting = iLevelSetting

		_createStrings()
		_createStrings = nil

		_G.PaperDollFrame:HookScript("OnShow", function(self)
			f:RegisterEvent("PLAYER_EQUIPMENT_CHANGED")
			f:RegisterEvent("ITEM_UPGRADE_MASTER_UPDATE")
			f:RegisterEvent("ARTIFACT_UPDATE")
			f:RegisterEvent("SOCKET_INFO_UPDATE")
			f:RegisterEvent("COMBAT_RATING_UPDATE")
			_updateItems("player", f)
			f:Show()
		end)

		_G.PaperDollFrame:HookScript("OnHide", function(self)
			f:UnregisterEvent("PLAYER_EQUIPMENT_CHANGED")
			f:UnregisterEvent("ITEM_UPGRADE_MASTER_UPDATE")
			f:UnregisterEvent("ARTIFACT_UPDATE")
			f:UnregisterEvent("SOCKET_INFO_UPDATE")
			f:UnregisterEvent("COMBAT_RATING_UPDATE")
			f:Hide()
		end)
	elseif event == "PLAYER_EQUIPMENT_CHANGED" or event == "ITEM_UPGRADE_MASTER_UPDATE"
	or event == "ARTIFACT_UPDATE" or event == "SOCKET_INFO_UPDATE" or event == "COMBAT_RATING_UPDATE" then
		if (...) == 16 then
			equiped[16] = nil
			equiped[17] = nil
		end
		_updateItems("player", f)
	elseif event == "INSPECT_READY" then
		_updateItems("target", g)
	end
end
f:SetScript("OnEvent", OnEvent)

SLASH_ILEVEL1 = "/ilevel"

SlashCmdList.ILEVEL = function(...)
	if (...) == "0" then
		setting = 0
		iLevelSetting = 0
		wipe(equiped)
		_updateItems("player", f)
		if g and _G.InspectPaperDollFrame:IsShown() then
			_updateItems("target", g)
		end
	elseif (...) == "1" then
		setting = 1
		iLevelSetting = 1
		wipe(equiped)
		_updateItems("player", f)
		if g and _G.InspectPaperDollFrame:IsShown() then
			_updateItems("target", g)
		end
	elseif (...) == "2" then
		setting = 2
		iLevelSetting = 2
		wipe(equiped)
		_updateItems("player", f)
		if g and _G.InspectPaperDollFrame:IsShown() then
			_updateItems("target", g)
		end
	end
	ChatFrame1:AddMessage("|cffff9999"..ADDON_NAME..":|r /ilevel ( 0 | 1 | 2 )\n 0 - Only show item levels.\n 1 - Show item levels and upgrades.\n 2 - Show item levels, upgrades and enchants and gems.")
	ChatFrame1:AddMessage("|cffff9999"..ADDON_NAME..":|r Current setting is " .. tostring(setting))
end
